package Net::LibNFS::X::Generic;

use strict;
use warnings;

=encoding utf-8

=head1 NAME

Net::LibNFS::X::Generic

=head1 DESCRIPTION

Errors of this class indicate errors with no other useful metadata.
Generally speaking, such errors are unrecoverable.

=cut

use parent 'Net::LibNFS::X::Base';

1;

